# Generated by Django 2.2.27 on 2022-04-12 18:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ci', '0026_job_start_end_time'),
    ]

    operations = [
        migrations.AlterField(
            model_name='backend',
            name='implementation_type',
            field=models.CharField(choices=[('fake', 'fake'), ('lava', 'lava'), ('null', 'null'), ('tuxsuite', 'tuxsuite')], default='null', max_length=64),
        ),
    ]
