# Generated by Django 2.2.24 on 2021-10-28 11:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0158_add_metric_comparison_to_projectstatus'),
    ]

    operations = [
        migrations.AlterField(
            model_name='metricthreshold',
            name='value',
            field=models.FloatField(blank=True, null=True),
        ),
    ]
