# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-08-02 18:35
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0052_recreate_projectstatus_data'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='projectstatus',
            name='previous',
        ),
    ]
