# Generated by Django 2.2.24 on 2021-10-27 16:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0155_add_build_and_environment_to_metric'),
    ]

    operations = [
        migrations.AlterField(
            model_name='metric',
            name='name',
            field=models.CharField(blank=True, default=None, max_length=256, null=True),
        ),
    ]
