# Generated by Django 2.2.24 on 2021-10-27 10:29

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0154_project_add_force_finishing_builds_field'),
    ]

    operations = [
        migrations.AddField(
            model_name='metric',
            name='build',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='metrics', to='core.Build'),
        ),
        migrations.AddField(
            model_name='metric',
            name='environment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='metrics', to='core.Environment'),
        ),
    ]
