# Generated by Django 2.2.24 on 2021-10-27 18:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0156_nullable_metric_name'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='metric',
            name='name',
        ),
    ]
